/*===========================================================================
 * 			       BEGIN DEFINES
 *=========================================================================*/

	/* Carriage Return ASCII Code - I use this mostly for seperating
	 * lines in Loading Files */
#define CR 10

	/* File access strings for fopen() and subsequently openfile() */
#define RO_TEXT "rt"
#define WO_TEXT "wt"
#define RO_BIN  "rb"
#define WO_BIN  "wb"

	/* The Version of the FILEUTIL.CPP File. With this I will be able to
	 * know if a reported bug has been fixed in newer versions or not.
	 * It's much easier to define this as a string than a number */
#define FILEUTIL_VERSION "0.20"

	/* The header string identifying the file. Currently this is only
	 * used in reporting errors in the general bug routine */
#define FILEUTIL_STRING "FILEUTIL.CPP v%s - January 1997, Dave Humphrey"

/*===========================================================================
 * 				END OF DEFINES
 *=========================================================================*/


/*===========================================================================
 *		BEGIN Function and Procedure Prototypes
 *
 * For better description of routines listed below, see the file FILEUTIL.CPP
 *=========================================================================*/

	/* Closes the log file...called when program exits */
void close_log_file (void);

	/* Returns TRUE if file can be opened for reading.  Otherwise
	 * returns FALSE */
boolean fileexists (const char *filename);

	/* Returns filesize in bytes of given filename.  Returns 0 if
	 * filename doesn't exist or is invalid */
long get_filesize(const char *filename);
long get_filesize(const FILE *f);

	/* Checks to see if a filename is a directory or not and change
	 * directory if it is one. Returns TRUE on success */
boolean isdirectory(const char far *string);

	/* Checks if a string has any '*' or '?' characters in it */
boolean iswildcard(const char far *string);

	/* Returns pointer to opened file on success, otherwise calls the
	 * file bug routine with an error */
FILE *openfile (const char *filename, const char *mode);

	/* Attempts to open the log file */
void open_log_file (const char *filename = "logfile.txt");

	/* Read file f until a CR, a EOF, or read string is longer than
	 * msl is encountered.  Returns FALSE if EOF was
	 * read, TRUE otherwise. */
boolean read_eol(FILE *f, char far *buf = NULL, const int msl = max_string_length);

	/* Reads entire file into memory, allocating and returning pointer
	 * to data.  Calls file_bug() and exits on error */
char far *read_file (const char *filename);

	/* Reads a 2 byte integer in standard format from a previously
	 * opened file (with fopen usually) */
int read_int (FILE *f);

	/* Reads 4 bytes from open file and converts to long integer */
long read_long (FILE *f);

	/* Read an open file f until a Space, CR or EOF is encountered.
	 * Returns FALSE if a EOF was read */
boolean read_word(FILE *f, char far *buf);

	/* Writes integer to open file */
boolean write_int (FILE *f, const unsigned i);
boolean write_long (FILE *f, const unsigned long l);

	/* Writes an Entry to the Log File, if Opened */
void write_log_entry (char *string = NULL, ...);

/*===========================================================================
 *		END Function and Procedure Prototypes
 *=========================================================================*/


/*===========================================================================
 *			BEGIN External Variables
 *
 * For a description of these variables, see the file FILEUTIL.CPP
 *=========================================================================== */
extern boolean log_file_open;
extern FILE *log_file;
/*===========================================================================
 * 			End of External Variables
 *=========================================================================*/