	/* Standard 'C' Includes */
#include <stdlib.h>
#include <stdio.h>
#include <conio.h>
#include <string.h>
#include <alloc.h>

	/* User Defined Includes */
#include "genutil.h"
#include "mouse.h"
#include "scrlbar.h"
#include "listbox.h"
#include "util.h"


/*========== Class DLINKED_LIST Constructor ===============================*/
void DLINKED_LIST::DLINKED_LIST (void) {
  next = prev = NULL;
  data = NULL;
 }
/*========== End of Class DLINKED_LIST Constructor ========================*/


/*========== Deletes Current Item from List ===============================*/
void DLINKED_LIST::~DLINKED_LIST (void) {
  if (next) next->prev = prev;
  if (prev) prev->next = next;
 }
/*========== End of procedure DLINKED_LIST::DLINKED_LIST() ================*/


/*========== Constructor for List_Box =====================================*/
void LIST_BOX::LIST_BOX (const int x1, const int y1, const int w, const int h,
			const int nc, const int nb, const int hc, const int hb) {
  x = x1;
  y = y1;
  width = w;
  height = h;
  norm_color = nc;
  high_color = hc;
  high_bg = hb;
  norm_bg = nb;
  active = FALSE;
  current = 0;
  view = start = list = end = NULL;
  scroll_bar.visible = FALSE;
  scroll_bar.x = x + width + 1;
  scroll_bar.y = y;
  scroll_bar.y50 = y;
  scroll_bar.length = height;
 }
/*========== End of List_Box constructor ==================================*/


/*========== Adds an item to end of the List ==============================*/
void LIST_BOX::add_end (const char far *string) {
  add_item (string);
 }
/*========== End of list class add_end() ==================================*/


/*========== Adds item to List ============================================*/
void LIST_BOX::add_item (const char far *string, DLINKED_LIST far *dllist) {
  DLINKED_LIST far *new_list;	/* Temp pointer */

	/* Create a new list item */
  if (!(new_list = new far DLINKED_LIST ))
      bug (MEM_ERR_MSG, "LIST_BOX::add_item() - *new_list (%d)", sizeof (DLINKED_LIST));

  if (!(new_list->data = new far char [_fstrlen(string) + 1]))
    bug (MEM_ERR_MSG, "LIST_BOX::add_item() - *new_list->data (%d)", _fstrlen(string) + 1);

  _fstrcpy (new_list->data, string);

	/* No list initially? Create one */
  if (!start) {
    scroll_bar.reset();
    start = new_list;
    view = start;
    end = start;
    list = start;
    scroll_bar.change_max(1);
    return;
   }

	/* Add new item to end of list */
  if (!dllist) {
    end->next = new_list;
    new_list->prev = end;
    end = new_list;
   }
  else if (!dllist->prev) {	/* Add to start of list */
    start->prev = new_list;
    new_list->next = start;
    start = new_list;
   }
  else {	/* Add somewhere in middle of list, just before dllist item */
    new_list->next = dllist;
    new_list->prev = dllist->prev;
    dllist->prev->next = new_list;
    dllist->prev = new_list;
   }

  scroll_bar.change_max(1);
 }
/*========== End of LIST_BOX::add_item() ==================================*/


/*========== Adds an item to start of the List ============================*/
void LIST_BOX::add_start (const char far *string) {
  add_item (string, start);
 }
/*========== End of list class add_start() ================================*/


/*========== Adds an item to a Sorted List ================================*/
void LIST_BOX::add_sort (const char far *string) {
  DLINKED_LIST far *temp_list;

	/* List Doesn't exist...so create it */
  if (!start) {
    add_item (string);
    return;
   }

	/* Find position to insert new item...sorted alphabetically */
  temp_list = start;

  while (temp_list) {
    if (_fstrcmp(string, temp_list->data) <= 0 ) {
      add_item (string, temp_list);
      return;
     }

    temp_list = temp_list->next;
   }

	/* Add to end of list */
  add_item (string);
 }
/*========== End of list class add_item() =================================*/


/*========== Adds  Special Items to end of the List =======================*/
void LIST_BOX::add_dual_end(const char far *string1, const char far *string2) {
  /* I wanted to use the same DLINKED List procedures but I had two string
   * variables...I overcame this by creating this procedure which adds the
   * 2nd after the 1st with a padding 0 or NULL character.  This way the first
   * one is seen only by printf and string routines while the 2nd is there
   * only for me. Neat...*/
  add_dual_item (string1, string2);
 }
/*========== End of list class LIST_BOX::add_dual_item() ==================*/


/*========== Adds Special Dual Item to List ===============================*/
void LIST_BOX::add_dual_item (const char far *string1, const char far *string2, DLINKED_LIST far *dllist) {
  DLINKED_LIST far *new_list;	/* Temp pointers */
  char far *temp_ptr;

	/* Create a new list item */
  if (!(new_list = new far DLINKED_LIST ))
      bug (MEM_ERR_MSG, "LIST_BOX::add_dual_item() - *new_list (%d)", sizeof (DLINKED_LIST));

  if (!(new_list->data = new far char[_fstrlen(string1) + _fstrlen(string2) + 3]))
    bug (MEM_ERR_MSG, "LIST_BOX::add_dual_item() - *new_list->data (%d)", _fstrlen(string1) + _fstrlen(string2) + 3);

  _fstrcpy (new_list->data, string1);
  chrcat (new_list->data, 0xFF);	/* Marker, so we can insert the 0 later */
  _fstrcat (new_list->data, string2);
  new_list->data[_fstrlen(new_list->data) + 1] = 0;
  temp_ptr = _fstrchr (new_list->data, 0xFF);
  *temp_ptr = 0;	/* Seperate the two strings */

	/* No list initially? Create one */
  if (!start) {
    scroll_bar.reset();
    start = new_list;
    view = start;
    end = start;
    list = start;
    scroll_bar.change_max(1);
    return;
   }

	/* Add new item to end of list */
  if (!dllist) {
    end->next = new_list;
    new_list->prev = end;
    end = new_list;
   }
  else if (!dllist->prev) {	/* Add to start of list */
    start->prev = new_list;
    new_list->next = start;
    start = new_list;
   }
  else {	/* Add somewhere in middle of list, just before dllist item */
    new_list->next = dllist;
    new_list->prev = dllist->prev;
    dllist->prev->next = new_list;
    dllist->prev = new_list;
   }

  scroll_bar.change_max(1);
 }
/*========== End of LIST_BOX::add_dual_item() =============================*/


/*========== Adds  Special Items to Sorted of the List ====================*/
void LIST_BOX::add_dual_sort(const char far *string1, const char far *string2) {
  /* I wanted to use the same DLINKED List procedures but I had two string
   * variables...I overcame this by creating this procedure which adds the
   * 2nd after the 1st with a padding 0 or NULL character.  This way the first
   * one is seen only by printf and string routines while the 2nd is there
   * only for me. Neat...*/
  DLINKED_LIST far *temp_list;

	/* List Doesn't exist...so create it */
  if (!start) {
    add_dual_item (string1, string2);
    return;
   }

	/* Find position to insert new item...sorted alphabetically */
  temp_list = start;

  while (temp_list) {
    if (_fstrcmp(string1, temp_list->data) <= 0 ) {
      add_dual_item (string1, string2, temp_list);
      return;
     }

    temp_list = temp_list->next;
   }

	/* Add to end of list */
  add_dual_item (string1, string2);
 }
/*========== End of list class LIST_BOX::add_dual_sort() ==================*/


/*========== Has there been a mouse click in the List Box? ================*/
int LIST_BOX::check (const int mx, const int my, const int event) {
  int i = 0;
  DLINKED_LIST far *temp_list;

  if (scroll_bar.visible && (i = scroll_bar.check (mx, my, event)) != NO_SCROLL) {
    if (i) {
      move(i);
      draw();
      _setcursortype (_NOCURSOR);
     }

    return (SCROLL_BAR);
   }
  else if (   mx >= x && ((mx <= x + width && scroll_bar.visible) || (mx <= x + width + 1 && !scroll_bar.visible))
	   && my >= y && my <= y + height + 1) {
	/* Make sure there is an item there under the mouse click */

    if (event != BUTTON_RELEASE) return (NO_ITEM);
    i = 0;
    temp_list = view;

    while (temp_list && i < my - y + 1) {
      i++;
      temp_list = temp_list->next;
     }

    if (i >= my - y + 1)
      return (my - y - current);
    else
      return(NO_ITEM);
   }
  else
    return (NO_ITEM);

 }
/*========== End of function LIST_BOX::check() ============================*/


/*========== Deletes an item from the list ================================*/
boolean LIST_BOX::delete_item (void) {

  if (!list) {	/* Make sure there is something to delete */
    return (FALSE);
   }
  else if (list->data) {	/* Delete data...if any */
    delete list->data;
    list->data = NULL;
   }

  if (!list->next && !list->prev) { /* Only one element in list */
    delete list;
    scroll_bar.reset ();
    view = end = start = list = NULL;
   }
  else if (!list->prev) {	/* Regular delete at start */
    start = start->next;
    delete start->prev;
    scroll_bar.change_max(-1);
   }
  else if (!list->next) { 	/* Regular delete at end */
    end = end->prev;
    delete end->next;
    scroll_bar.change_max(-1);
   }
  else { 	/* Arbitrary delete somewhere in the list */
    list = list->prev;
    delete list->prev;
    scroll_bar.change_max(-1);
   }

  return (TRUE);
 }
/*========== End of the list class del_item() =============================*/


/*========== Deletes the entire list ======================================*/
void LIST_BOX::delete_list (void) {

  scroll_bar.visible = FALSE;

  do {
    list = end;
   } while (list && delete_item());

 }
/*========== End of the routine LIST_BOX::delete_list() ===================*/


/*========== Deletes the currently selected item from list ================*/
boolean LIST_BOX::delete_select (void) {
  DLINKED_LIST far *temp_list;
  int count = 0;

  if (!view) return (FALSE);
  temp_list = view;

  while (temp_list->next && count != current) {
    temp_list = temp_list->next;
    count++;
   }

  if (view == temp_list && temp_list->next)
    view = temp_list->next;
  else if (view == temp_list)
    view = temp_list->prev;
  else
    return (FALSE);

  list = temp_list;
  delete_item();
  temp_list = view;
  count = 0;

  while (temp_list->next && count != current) {
    temp_list = temp_list->next;
    count++;
   }

  if (current > count) current = count;
  return(TRUE);
 }
/*========== End of function del_current() ================================*/


/*========== Draws the List on the Screen...Text Only =====================*/
void LIST_BOX::draw (void) {
  int count;
  boolean full = TRUE;
  DLINKED_LIST far *temp_list;

  hide_mouse ();
  _setcursortype(_NOCURSOR);  /* Turn the cursor off */
  temp_list = view;
  textcolor (norm_color);
  textbackground (norm_bg);

  for (count = 0; count < height; count++) {
    gotoxy(x, y + count);

    if (temp_list) { /* Do we have a valid item? */
      if (current == count && active) {
	textcolor (high_color);
	textbackground (high_bg);
       }

      cprintf (" %-*s ", width, temp_list->data);

      if (current == count && active) {
	textcolor (norm_color);
	textbackground (norm_bg);
       }

      temp_list = temp_list->next;
     } /* End of valid item */
    else { /* Not-Valid Item...just print blank line... */
      cprintf ("%-*s", width + 2, " ");
      full = FALSE;
     }
   } /* End for loop */


  if ((full && temp_list && temp_list->next) || (view && view->prev)) {
    scroll_bar.visible = TRUE;
    scroll_bar.draw();
   }
  else
    scroll_bar.visible = FALSE;

  _setcursortype(_NORMALCURSOR);  /* Turn the cursor off */
  show_mouse ();
 }
/*========== End of function LIST_BOX::draw() =============================*/


/*========== Attempts to move the current selected ========================*/
boolean LIST_BOX::move(int where) {
  int count, cview;
  DLINKED_LIST far *temp_list;
  where += current;

  if (where == current)
    return(FALSE);
  else if (where > current) {
    while (current < height - 1 && current < where)
      current++;

    if (where != current) {
      count = current;

      while (count < where && view->next) {
	count++;
	view = view->next;
       }
     }

    temp_list = view;

    for (count = 0; count < current && temp_list->next; count++) {
      temp_list = temp_list->next;
     }

    current = count;
   }
  else if (where < current) {
    while (current > 0 && current > where)
      current--;

    if (where != current) {
      count = current;

      while (count > where && view->prev) {
	count--;
	view = view->prev;
       }
     }

    temp_list = view;

    for (count = 0; count < current && temp_list->next; count++) {
      temp_list = temp_list->next;
     }

    current = count;
   }

	/* Update the position on the scroll bar */
  if (scroll_bar.visible) {
    temp_list = start;
    count = 0;
    cview = 0;

    while (temp_list) {
      if (temp_list == view) {
	cview = count + current;
	break;
       }

      count++;
      temp_list = temp_list->next;
     }

    scroll_bar.change_value (cview);
   }

  return(TRUE);
 }
/*========== End of function move() =======================================*/


/*========== Returns the Name at the current position =====================*/
boolean LIST_BOX::get_name(char far *string) {
  DLINKED_LIST far *temp_list;
  int count;

  temp_list = view;

  for (count = 0; count < current && temp_list->next; count++) {
    temp_list = temp_list->next;
   }

  if (count == current)
    if (!string)	/* Return pointer to element */
      list = temp_list;
    else {	/* Return string, only the maximum length of string though */
      _fstrncpy (string, temp_list->data, MAX_STRING_LENGTH);
      string[MAX_STRING_LENGTH] = 0;	/* Make sure string is NULL terminated */
     }
  else
    return(FALSE);

  return(TRUE);
 }
/*========== End of function get_name() ===================================*/


/*========== Gets the Extended Value...If Any =============================*/
unsigned char get_ext_char (const DLINKED_LIST far *list) {
  if (list && list->data)
    return (list->data[strlen(list->data) + 1]);
  else
    return (-1);
 }
/*========== End of function LIST_BOX::get_ext_int() ======================*/


/*========== Gets the Extended Value...If Any =============================*/
unsigned int get_ext_int (const DLINKED_LIST far *list) {
  if (list && list->data)
    return ((unsigned int)strtol(list->data + strlen(list->data) + 1, NULL, 0) );
  else
    return (-1);
 }
/*========== End of function LIST_BOX::get_ext_int() ======================*/


/*========== Gets the Extended Value...If Any =============================*/
unsigned long get_ext_long (const DLINKED_LIST far *list) {
  if (list && list->data)
    return (strtoul(list->data + strlen(list->data) + 1, NULL, 0) );
  else
    return (0);
 }
/*========== End of function LIST_BOX::get_ext_long() =====================*/